--[[
Copyright (C) GtX (Andy), 2018

Interface: 1.5.3.1 b1848

Author: GtX	 @  LS-Modcompany
Date: 08.09.2018

Support only at: https://ls-modcompany.com

History:
Version: 1.0.0.0 - Release Edition @ 08.09.2018

Important:
No changes are to be made to this script without permission from GtX @ https://ls-modcompany.com
This includes converting to other game versions before or after FS/LS17

** PLACEABLE ONLY! **
]]

Letterbox = {};
Letterbox.ModDir = g_currentModDirectory;
Letterbox_mt = Class(Letterbox, Placeable);

-- File paths NOT to be changed as stated above!!!!
-- This is the correct format Giants use and how it should be ;-).
source(Utils.getFilename("scripts/LetterboxGui.lua", Letterbox.ModDir));
source(Utils.getFilename("scripts/LetterboxEvent.lua", Letterbox.ModDir));

InitObjectClass(Letterbox, "Letterbox");

function Letterbox:new(isServer, isClient, customMt)
	local mt = customMt;
	if mt == nil then
		mt = Letterbox_mt;
	end;

	local self = Placeable:new(isServer, isClient, mt);

	registerObjectClassName(self, "Letterbox");

	local i = 0;
	local modDescXML = loadXMLFile("modDesc", Letterbox.ModDir .. "modDesc.xml");
	while true do
		local text = string.format("modDesc.l10n.text(%d)", i);
		if not hasXMLProperty(modDescXML, text) then
			break;
		end;
		local name = getXMLString(modDescXML, text.."#name");
		g_i18n.globalI18N.texts[name] = g_i18n:getText(name);
		i = i + 1;
	end;

	return self;
end;

function Letterbox:load(xmlFilename, x,y,z, rx,ry,rz, initRandom)
	if not Letterbox:superClass().load(self, xmlFilename, x,y,z, rx,ry,rz, initRandom) then
		return false;
	end;

	local xmlFile = loadXMLFile("TempXML", xmlFilename);

	self.letterDisplay = Utils.indexToObject(self.nodeId, getXMLString(xmlFile, "placeable.letterbox.displays#letterDisplay"));
	self.numberDisplay = Utils.indexToObject(self.nodeId, getXMLString(xmlFile, "placeable.letterbox.displays#numberDisplay"));
	self.playerTrigger = Utils.indexToObject(self.nodeId, getXMLString(xmlFile, "placeable.letterbox.playerTrigger#index"));

	self.letterBoxFlagId = Utils.indexToObject(self.nodeId, getXMLString(xmlFile, "placeable.letterbox.flag#index"));
	if self.letterBoxFlagId ~= nil then
		local flagUp = getXMLString(xmlFile, "placeable.letterbox.flag#rotUp");
		self.letterBoxFlagUp = Utils.getNoNil(Utils.getRadiansFromString(flagUp, 3), {getRotation(self.letterBoxFlagId)});
		local flagDown = getXMLString(xmlFile, "placeable.letterbox.flag#rotDown");
		self.letterBoxFlagDown = Utils.getNoNil(Utils.getRadiansFromString(flagDown, 3), {getRotation(self.letterBoxFlagId)});
	end;

	self.boxColourIndex = 0;
	local i = 0;
	while true do
		local key = string.format("placeable.letterbox.meshColourNodes.colourNode(%d)", i);
		if not hasXMLProperty(xmlFile, key) then
			break;
		end;

		local node = Utils.indexToObject(self.nodeId, getXMLString(xmlFile, key .. "#node"));
		if node ~= nil then
			if getHasClassId(node, ClassIds.SHAPE) then
				if getHasShaderParameter(node, "colorScale") then
					if self.letterBoxColourNodes == nil then
						self.letterBoxColourNodes = {};
						self.boxColourIndex = Utils.getNoNil(getXMLInt(xmlFile, "placeable.letterbox.meshColourNodes#defaultColourIndex"), 8);
					end;
					self.letterBoxColourNodes[i + 1] = node;
				else
					print("ERROR: [Letterbox.lua] 'vehicleShader' not found at node '"..getName(node).."'! 'meshColourNode' could not be set.");
				end;
			else
				print("Warning: [Letterbox.lua] Could not set mesh colour to '"..getName(node).."' because node is not a shape!");
			end;
		end;
		i = i + 1;
	end;

	-- Allow the texts to be changed in each 'xml' so the script can be used for signs etc.
	local boxColour = g_i18n:getText(Utils.getNoNil(getXMLString(xmlFile, "placeable.letterbox.texts#boxColour"), "Letterbox_BoxColour"));
	local textColour = g_i18n:getText(Utils.getNoNil(getXMLString(xmlFile, "placeable.letterbox.texts#textColour"), "Letterbox_TextColour"));
	local numberColour = g_i18n:getText(Utils.getNoNil(getXMLString(xmlFile, "placeable.letterbox.texts#numberColour"), "Letterbox_NumberColour"));
	local flagDown = g_i18n:getText(Utils.getNoNil(getXMLString(xmlFile, "placeable.letterbox.texts#flagDown"), "Letterbox_FlagDown"));
	local flagUp = g_i18n:getText(Utils.getNoNil(getXMLString(xmlFile, "placeable.letterbox.texts#flagUp"), "Letterbox_FlagUp"));
	self.letterboxTexts = {boxColour = boxColour, textColour = textColour, numberColour = numberColour, flagDown = flagDown, flagUp = flagUp};

	local canLoad = true;
	if self.letterDisplay == nil or self.numberDisplay == nil or self.playerTrigger == nil then
		canLoad = false;
		print("ERROR: [Letterbox.lua] Something went wrong. Please check that the placeable has the following. 'letterDisplay', 'numberDisplay', 'playerTrigger'");
	end;

	delete(xmlFile);

	return canLoad;
end;

function Letterbox:finalizePlacement(x, y, z, rx, ry, rz, initRandom)
	Letterbox:superClass().finalizePlacement(self);

	self.canInteract = false;
	self.numberColourIndex = 8;
	self.textColourIndex = 8;
	self.flagState = 0;

	self.guiNumberTable = {1, 1, 1, 1};
	self.guiLetterTable = {33, 33, 33, 33, 33, 33, 33, 33, 33, 33, 33, 33, 33, 33};

	self.letterTable = {};
	self.letterTable[1] = {letter = "A", posX = 1, posY = 9};
	self.letterTable[2] = {letter = "B", posX = 1, posY = 8};
	self.letterTable[3] = {letter = "C", posX = 1, posY = 7};
	self.letterTable[4] = {letter = "D", posX = 1, posY = 6};
	self.letterTable[5] = {letter = "E", posX = 1, posY = 5};
	self.letterTable[6] = {letter = "F", posX = 1, posY = 4};
	self.letterTable[7] = {letter = "G", posX = 1, posY = 3};
	self.letterTable[8] = {letter = "H", posX = 1, posY = 2};
	self.letterTable[9] = {letter = "I", posX = 1, posY = 1};
	self.letterTable[10] = {letter = "J", posX = 1, posY = 0};
	self.letterTable[11] = {letter = "K", posX = 1, posY = -1};
	self.letterTable[12] = {letter = "L", posX = 2, posY = 9};
	self.letterTable[13] = {letter = "M", posX = 2, posY = 8};
	self.letterTable[14] = {letter = "N", posX = 2, posY = 7};
	self.letterTable[15] = {letter = "O", posX = 2, posY = 6};
	self.letterTable[16] = {letter = "P", posX = 2, posY = 5};
	self.letterTable[17] = {letter = "Q", posX = 2, posY = 4};
	self.letterTable[18] = {letter = "R", posX = 2, posY = 3};
	self.letterTable[19] = {letter = "S", posX = 2, posY = 2};
	self.letterTable[20] = {letter = "T", posX = 2, posY = 1};
	self.letterTable[21] = {letter = "U", posX = 2, posY = 0};
	self.letterTable[22] = {letter = "V", posX = 2, posY = -1};
	self.letterTable[23] = {letter = "W", posX = 3, posY = 9};
	self.letterTable[24] = {letter = "X", posX = 3, posY = 8};
	self.letterTable[25] = {letter = "Y", posX = 3, posY = 7};
	self.letterTable[26] = {letter = "Z", posX = 3, posY = 6};
	self.letterTable[27] = {letter = "Ü", posX = 3, posY = 5};
	self.letterTable[28] = {letter = "Ä", posX = 3, posY = 4};
	self.letterTable[29] = {letter = "Ö", posX = 3, posY = 3};
	self.letterTable[30] = {letter = "ẞ", posX = 3, posY = 2};
	self.letterTable[31] = {letter = "-", posX = 3, posY = 1};
	self.letterTable[32] = {letter = ".", posX = 3, posY = 0};
	self.letterTable[33] = {letter = " ", posX = 3, posY = -1};

	if self.letterDisplay ~= nil then
		local letterCount = getNumOfChildren(self.letterDisplay);
		if letterCount >= 14 then
			local firstChild = getChildAt(self.letterDisplay, 0);
			if getHasShaderParameter(firstChild, "textColour") then
				self:setLetterShader(self.letterDisplay, self.guiLetterTable, self.textColourIndex);
			else
				print(string.format("ERROR: [Letterbox.lua] - Correct 'textShader' was not found on children at '%s'", getName(self.letterDisplay)));
			end;
		else
			print(string.format("ERROR: [Letterbox.lua] - 'letterDisplay' %s only contains %d children! 14 are required.", getName(self.letterDisplay), letterCount));
		end;
	end;

	if self.numberDisplay ~= nil then
		local numCount = getNumOfChildren(self.numberDisplay);
		if numCount >= 4 then
			local firstChild = getChildAt(self.numberDisplay, 0);
			if getHasShaderParameter(firstChild, "numberColor") then
				self:setNumberShader(self.numberDisplay, self.guiNumberTable, self.numberColourIndex);
			else
				print(string.format("ERROR: [Letterbox.lua] - Correct 'numberShader' was not found on children at '%s'", getName(self.numberDisplay)));
			end;
		else
			print(string.format("ERROR: [Letterbox.lua] - 'numberDisplay' %s only contains %d children! 4 are required.", getName(self.numberDisplay), numCount));
		end;
	end;

	if self.letterBoxFlagId ~= nil then
		self.flagState = 1;
		self.letterBoxFlag = {};
		self.letterBoxFlag.node = self.letterBoxFlagId;
		self.letterBoxFlag.state = {self.letterBoxFlagDown, self.letterBoxFlagUp};
		setRotation(self.letterBoxFlag.node, unpack(self.letterBoxFlagDown));
	end;

	if self.letterBoxColourNodes ~= nil then
		local r, g, b, a = unpack(g_vehicleColors[self.boxColourIndex]);
		for _, node in pairs (self.letterBoxColourNodes) do
			setShaderParameter(node, "colorScale", r, g, b, a, false);
			local x, _, z, w = getShaderParameter(node, "RDT");
			setShaderParameter(node, "RDT", x, 0, z, w, false);
		end;
	end;

	if self.playerTrigger ~= nil then
		addTrigger(self.playerTrigger,"playerTriggerCallback", self);
	else
		print("ERROR: [Letterbox.lua] - No 'playerTrigger' found.");
	end;

	return true;
end;

function Letterbox:delete()
	unregisterObjectClassName(self);
	Letterbox:superClass().delete(self);

	if self.playerTrigger ~= nil then
		removeTrigger(self.playerTrigger);
	end;
end;

function Letterbox:readStream(streamId, connection)
	Letterbox:superClass().readStream(self, streamId, connection);

	guiLetterTable = {};
	for i = 1, 14 do
		guiLetterTable[i] = streamReadInt8(streamId);
	end;

	guiNumberTable = {};
	for i = 1, 4 do
		guiNumberTable[i] = streamReadInt8(streamId);
	end;

	local numberColour = streamReadInt8(streamId);
	local textColour = streamReadInt8(streamId);
	local flagState = streamReadInt8(streamId);
	local boxColour = streamReadInt8(streamId);

	self:setLetterboxData(guiLetterTable, guiNumberTable, numberColour, textColour, flagState, boxColour, true);
end

function Letterbox:writeStream(streamId, connection)
	Letterbox:superClass().writeStream(self, streamId, connection);

	for i = 1, 14 do
		streamWriteInt8(streamId, self.guiLetterTable[i]);
	end;

	for i = 1, 4 do
		streamWriteInt8(streamId, self.guiNumberTable[i]);
	end;

	streamWriteInt8(streamId, self.numberColourIndex);
	streamWriteInt8(streamId, self.textColourIndex);
	streamWriteInt8(streamId, self.flagState);
	streamWriteInt8(streamId, self.boxColourIndex);
end

function Letterbox:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
	if not Letterbox:superClass().loadFromAttributesAndNodes(self, xmlFile, key, resetVehicles) then
		return false;
	end;

	local guiLetterTable = {};
	local guiNumberTable = {};

	local letterIds = getXMLString(xmlFile, key .. ".ids#letterIds");
	if letterIds ~= nil then
		local ids = Utils.splitString(" ", letterIds);
		if #ids == 14 then
			for l = 1, 14 do
				local id = tonumber(ids[l]);
				table.insert(guiLetterTable, id);
				l = l + 1;
			end;
		end;
	else
		guiLetterTable = self.guiLetterTable;
	end;

	local numberIds = getXMLString(xmlFile, key .. ".ids#numberIds");
	if numberIds ~= nil then
		local ids = Utils.splitString(" ", numberIds);
		if #ids == 4 then
			for n = 1, 4 do
				local id = tonumber(ids[n]);
				table.insert(guiNumberTable, id);
				n = n + 1;
			end;
		end;
	else
		guiNumberTable = self.guiNumberTable;
	end;

	local getTextColour = Utils.getNoNil(getXMLInt(xmlFile, key..".colourIndexs#text"), 1);
	local textColour = math.min(getTextColour, #g_vehicleColors);

	local getNumberColour = Utils.getNoNil(getXMLInt(xmlFile, key..".colourIndexs#number"), 1);
	local numberColour = math.min(getNumberColour, #g_vehicleColors);

	local boxColour = 0;
	if self.letterBoxColourNodes ~= nil then
		local getBoxColour = Utils.getNoNil(getXMLInt(xmlFile, key..".colourIndexs#box"), boxColourBackup);
		boxColour = math.min(getBoxColour, #g_vehicleColors);
	end;

	local flagState = 0;
	if self.letterBoxFlag ~= nil then
		flagState = Utils.getNoNil(getXMLInt(xmlFile, key..".flagState#id"), 1);
		if flagState < 1 or flagState > 2 then
			flagState = 1;
		end;
	end;

	self:setLetterboxData(guiLetterTable, guiNumberTable, numberColour, textColour, flagState, boxColour);

	return true;
end;

function Letterbox:getSaveAttributesAndNodes(nodeIdent)
	local attributes, nodes = Letterbox:superClass().getSaveAttributesAndNodes(self, nodeIdent);

	local l = 0;
	local letterIds, text = "", "";
	for _, id in pairs(self.guiLetterTable) do
		if l > 0 then
			letterIds = letterIds .. " ";
		end;

		letterIds = letterIds .. id;
		text = text .. self.letterTable[id].letter;

		l = l + 1;
	end;

	local n = 0;
	local numberIds, numbers = "", "";
	for _, id in pairs(self.guiNumberTable) do
		if n > 0 then
			numberIds = numberIds .. " ";
		end;

		numberIds = numberIds .. id;

		local num = id - 2
		if num == -1 then
			num = " ";
		end;
		numbers = numbers .. num;

		n = n + 1;
	end;

	nodes = nodes..nodeIdent..' <displayData text="'..text..'" numbers="'..numbers..'" />'; -- This is just to make it easy to identify in the savegame if needed.

	nodes = nodes.."\n"..nodeIdent..' <ids letterIds="'..letterIds..'" numberIds="'..numberIds..'" />';

	nodes = nodes.."\n"..nodeIdent..' <colourIndexs text="'..self.textColourIndex..'" number="'..self.numberColourIndex..'" box="'..self.boxColourIndex..'" />';

	nodes = nodes.."\n"..nodeIdent..' <flagState id="'..self.flagState..'" />';

	return attributes, nodes;
end;

function Letterbox:update(dt)
	if self.canInteract then
		g_currentMission:addHelpButtonText(g_i18n:getText("input_Letterbox_Menu"), InputBinding.Letterbox_Menu);
		if InputBinding.hasEvent(InputBinding.Letterbox_Menu) then
			if g_letterboxGui == nil then
				g_letterboxGui = LetterboxGui:new();
				g_gui:loadGui(Letterbox.ModDir .. "scripts/LetterboxGui.xml", "LetterboxGui", g_letterboxGui);
			end;
			self:openLetterboxMenu();
		end;

		if g_currentMission.controlledVehicle ~= nil then
			self.canInteract = false;
		end;
	end;
end;

function Letterbox:draw()
end;

function Letterbox:mouseEvent(posX, posY, isDown, isUp, button)
end;

function Letterbox:keyEvent(unicode, sym, modifier, isDown)
end;

function Letterbox:playerTriggerCallback(triggerId, otherId, onEnter, onLeave, onStay)
	if (g_currentMission.controlPlayer and g_currentMission.player and otherId == g_currentMission.player.rootNode) then
		if (onEnter) then
			self.canInteract = true;
		elseif (onLeave) then
			self.canInteract = false;
		end;
	end;
end;

function Letterbox:openLetterboxMenu()
	local gui = g_gui:showDialog("LetterboxGui");
	if gui ~= nil then
		gui.target:setTitle(g_i18n:getText("Letterbox_GUI_Title"));
		gui.target:sendTableData(self.guiLetterTable, self.guiNumberTable, self.numberColourIndex, self.textColourIndex, self.flagState, self.boxColourIndex, self.letterboxTexts);
		gui.target:loadCallback(self.setLetterboxData, self);
	end;
end;

function Letterbox:setLetterboxData(guiLetterTable, guiNumberTable, numberColour, textColour, flagState, boxColour, noEventSend)
	LetterboxEvent.sendEvent(self, guiLetterTable, guiNumberTable, numberColour, textColour, flagState, boxColour, noEventSend);

	self.guiLetterTable = guiLetterTable;
	self.guiNumberTable = guiNumberTable;
	self.numberColourIndex = numberColour;
	self.textColourIndex = textColour;
	self.flagState = math.min(flagState, 2);
	self.boxColourIndex = boxColour;

	if g_currentMission:getIsClient() then
		if self.letterBoxColourNodes ~= nil then
			for _, node in pairs (self.letterBoxColourNodes) do
				local r, g, b, a = unpack(g_vehicleColors[boxColour]);
				setShaderParameter(node, "colorScale", r, g, b, a, false);
			end;
		end;

		if self.letterBoxFlag ~= nil then
			setRotation(self.letterBoxFlag.node, unpack(self.letterBoxFlag.state[self.flagState]))
		end;

		if self.numberDisplay ~= nil then
			self:setNumberShader(self.numberDisplay, self.guiNumberTable, numberColour);
		end;

		if self.letterDisplay ~= nil then
			self:setLetterShader(self.letterDisplay, self.guiLetterTable, textColour);
		end;
	end;
end;

function Letterbox:setLetterShader(index, textTable, colourIndex)
	if index ~= nil then
		local r, g, b, a = unpack(g_vehicleColors[colourIndex]);

		for i = 1, 14 do
			local child = getChildAt(index, i - 1);
			local letterId = textTable[i];

			if letterId < 0 or letterId > 33 then
				letterId = 33;
			end;

			setShaderParameter(child, "posX", self.letterTable[letterId].posX, 0, 0, 0, false);
			setShaderParameter(child, "posY", self.letterTable[letterId].posY, 0, 0, 0, false);
			setShaderParameter(child, "textColour", r, g, b, a, false);
		end;
	end;
end;

function Letterbox:setNumberShader(index, numberTable, colourIndex)
	if index ~= nil then
		local r, g, b, a = unpack(g_vehicleColors[colourIndex]);

		for i = 1, 4 do
			local child = getChildAt(index, i - 1);
			local numberId = numberTable[i] - 2;

			if numberId < -1 or numberId > 9 then
				numberId = -1;
			end;

			setShaderParameter(child, "number", numberId, 0, 0, 0, false);
			setShaderParameter(child, "numberColor", r, g, b, a, false);
		end;
	end;
end;

registerPlaceableType("Letterbox", Letterbox);